/**
 * 
 */
package com.github.ansell.oas.data.test;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentMap;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;
import org.openrdf.model.BNode;
import org.openrdf.model.Literal;
import org.openrdf.model.Model;
import org.openrdf.model.URI;
import org.openrdf.model.Value;
import org.openrdf.model.impl.LinkedHashModel;
import org.openrdf.model.util.GraphUtil;
import org.openrdf.model.vocabulary.OWL;
import org.openrdf.model.vocabulary.RDF;
import org.openrdf.model.vocabulary.RDFS;
import org.openrdf.query.BindingSet;
import org.openrdf.query.QueryLanguage;
import org.openrdf.query.TupleQuery;
import org.openrdf.query.TupleQueryResult;
import org.openrdf.repository.Repository;
import org.openrdf.repository.RepositoryConnection;
import org.openrdf.repository.sail.SailRepository;
import org.openrdf.rio.RDFFormat;
import org.openrdf.rio.RDFParseException;
import org.openrdf.rio.RDFParser;
import org.openrdf.rio.Rio;
import org.openrdf.rio.UnsupportedRDFormatException;
import org.openrdf.rio.helpers.StatementCollector;
import org.openrdf.sail.memory.MemoryStore;
import org.restlet.data.Status;
import org.restlet.resource.ResourceException;
import org.restlet.security.User;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.github.ansell.oas.data.DataHandler;
import com.github.ansell.oas.objects.Annotation;
import com.github.ansell.oas.objects.Ontology;
import com.github.ansell.oas.objects.ResourceCount;
import com.github.ansell.oas.objects.SearchResult;
import com.github.ansell.oas.test.AbstractTestUtils;
import com.github.ansell.oas.utils.OasConstants;
import com.github.ansell.oas.webservice.BaseOasActions;

/**
 * An abstract test case for the DataHandler interface to test the functionality behind the web
 * service API without testing the web service code.
 * 
 * @author Peter Ansell p_ansell@yahoo.com
 */
public abstract class AbstractDataHandlerTest
{
    private final Logger log = LoggerFactory.getLogger(this.getClass());
    
    private DataHandler dataHandler;
    
    private String testUriString;
    private String testLiteralString;
    private URI testDatatype;
    private String testLanguage;
    
    private URI testUniqueAnnotationId;
    private URI testUniqueAnnotationId2;
    private URI testAnnotatedObjectUri;
    private URI testAnnotatedObjectType;
    private URI testAnnotator;
    private URI testOntologyUri1;
    private Literal testTagLiteral;
    private Literal testLongTagLiteral;
    private URI differentTestOntologyUri;
    
    private URI testInvalidAnnotationId;
    private URI testInvalidAnnotatedObjectType;
    private URI testInvalidAnnotatedObjectUri;
    private URI testInvalidOntologyTermUri;
    
    private URI testOntologyVersionUri1;
    private URI testOntologyUri2;
    private URI testOntologyVersionUri2;
    private URI testInferredOntologyUri1;
    
    private URI testOntologyTermLabel;
    private URI testOntologyDescendant;
    private URI testOntologyDescendantsChild;
    
    private final boolean deleteOldVersionsFalse = false;
    private final boolean deleteOldVersionsTrue = true;
    
    private String testOntologyVersionUriBase;
    
    private final boolean recursiveFalse = false;
    private final boolean recursiveTrue = true;
    
    private final boolean childrenCountFalse = false;
    private final boolean childrenCountTrue = true;
    
    private void checkCounts(final Ontology nextOntology, final URI currentParent, final int nextExpectedRecursive,
            final int nextExpectedDirect, final int depth, final List<URI> currentPath) throws Exception
    {
        Assert.assertFalse("Found a cycle: " + currentPath.toString(), currentPath.contains(currentParent));
        final List<URI> nextPath = new ArrayList<URI>(currentPath);
        nextPath.add(currentParent);
        
        Assert.assertFalse("Likely found a cycle: " + nextPath.toString(), depth > 20);
        
        final Model result =
                this.downloadOntologySubsetToModel(nextOntology, currentParent, this.recursiveFalse,
                        this.childrenCountTrue);
        
        final Model filtered = result.filter(currentParent, RDFS.SEEALSO, null);
        // DebugUtils.printStatements(filtered);
        
        final int directCountCheck = filtered.objects().size();
        int recursiveCountCheck = filtered.objects().size();
        
        for(final Value nextChild : filtered.objects())
        {
            Assert.assertTrue(nextChild instanceof URI);
            final Model child = result.filter((URI)nextChild, null, null);
            // DebugUtils.printStatements(child);
            
            final Literal directCount =
                    child.filter((URI)nextChild, OasConstants.OAS_URI_DIRECT_COUNT, null).objectLiteral();
            final Literal recursiveCount =
                    child.filter((URI)nextChild, OasConstants.OAS_URI_RECURSIVE_COUNT, null).objectLiteral();
            
            Assert.assertTrue(recursiveCount.intValue() >= directCount.intValue());
            
            recursiveCountCheck += recursiveCount.intValue();
            this.checkCounts(nextOntology, (URI)nextChild, recursiveCount.intValue(), directCount.intValue(),
                    depth + 1, nextPath);
        }
        
        // Actual recursive count will be greater than expected if there are multiple paths to
        // children
        Assert.assertTrue("Recursive count check failed for: " + currentParent,
                nextExpectedRecursive <= recursiveCountCheck);
        Assert.assertEquals("Direct count check failed for: " + currentParent, nextExpectedDirect, directCountCheck);
    }
    
    private Model downloadOntologySubsetToModel(final Ontology nextOntology, final URI parentUri,
            final boolean recursive, final boolean includeChildrenCount) throws RDFParseException,
        UnsupportedRDFormatException, IOException
    {
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                parentUri, recursive, includeChildrenCount);
        
        return Rio.parse(new ByteArrayInputStream(outputStream.toByteArray()), "", RDFFormat.RDFJSON);
    }
    
    /**
     * Each invocation of this method returns a new instance of an implementation of the Annotation
     * interface.
     * 
     * @return a new Annotation instance for each invocation.
     */
    public abstract Annotation getNewAnnotation();
    
    /**
     * Each invocation of this method returns a new instance of an implementation of the DataHandler
     * interface. The DataHandler must be empty when returned from this method.
     * 
     * @return a new DataHandler instance for each invocation.
     */
    public abstract DataHandler getNewDataHandler();
    
    /**
     * Each invocation of this method returns a new instance of an implementation of the Ontology
     * interface.
     * 
     * @return a new Ontology instance for each invocation.
     */
    public abstract Ontology getNewOntology();
    
    /**
     * Creates variables before each test.
     * 
     * @throws java.lang.Exception
     */
    @Before
    public void setUp() throws Exception
    {
        this.dataHandler = this.getNewDataHandler();
        
        this.testUriString = "http://my.name/test.rdf#me";
        this.testLiteralString = "happy bear";
        this.testDatatype = this.dataHandler.createURI("http://my.domain.org/testdatatype");
        this.testLanguage = "en_AU";
        
        this.testUniqueAnnotationId =
                this.dataHandler.createURI("http://example.org/annotationwebapp/annotation/id/12555");
        this.testUniqueAnnotationId2 =
                this.dataHandler.createURI("http://example.org/annotationwebapp/annotation/id/anotherid2");
        this.testAnnotatedObjectUri = this.dataHandler.createURI("http://example.net/objects/byid/09876");
        this.testAnnotatedObjectType = this.dataHandler.createURI("http://example.purl.org.au/vocab/VC_000113");
        this.testAnnotator = this.dataHandler.createURI("http://example.net/annotationwebapp/authors/annotator101");
        
        this.testOntologyUri1 = this.dataHandler.createURI("http://example.purl.org.au/domainontology/JB_000007");
        this.testOntologyUri2 = this.dataHandler.createURI("http://test.example.org/ontology/0139");
        
        this.testOntologyVersionUriBase = "http://example.purl.org.au/domainontology/version:";
        this.testOntologyVersionUri1 =
                this.dataHandler.createURI("http://example.purl.org.au/domainontology/version:1");
        this.testOntologyVersionUri2 = this.dataHandler.createURI("http://test.example.org/ontology/0139/version:2");
        
        this.testInferredOntologyUri1 = this.dataHandler.createURI("http://test.purl.org.au/inferred/ontology/1");
        
        this.differentTestOntologyUri =
                this.dataHandler.createURI("http://other.example.purl.org/otherdomain/KA_311411");
        
        this.testTagLiteral = this.dataHandler.createLiteral("catch 22");
        
        this.testLongTagLiteral =
                this.dataHandler.createLiteral("The catch 22 is that there is no way of knowing what it is.");
        
        this.testInvalidAnnotationId =
                this.dataHandler.createURI("http://fake.example.org/annotationwebapp/annotation/fakeid/123");
        this.testInvalidAnnotatedObjectType =
                this.dataHandler.createURI("http://fake.purl.org/vocabulary/fake/FAKE_000911");
        this.testInvalidAnnotatedObjectUri =
                this.dataHandler
                        .createURI("http://fake.example.org/annotationwebapp/vocabulary/fakeobjects_inc/PWN_1337");
        this.testInvalidOntologyTermUri =
                this.dataHandler.createURI("http://fake.purl.org/vocabulary/otherontology/OTHR_135792468");
        
        this.testOntologyTermLabel = this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0000001");
        
        this.testOntologyDescendant = this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025131");
        
        this.testOntologyDescendantsChild = this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0009007");
    }
    
    /**
     * Cleans up the variables after each test.
     * 
     * @throws java.lang.Exception
     */
    @After
    public void tearDown() throws Exception
    {
        this.dataHandler = null;
        
        this.testUriString = null;
        this.testLiteralString = null;
        this.testDatatype = null;
        this.testLanguage = null;
        
        this.testUniqueAnnotationId = null;
        this.testUniqueAnnotationId2 = null;
        this.testAnnotatedObjectUri = null;
        this.testAnnotatedObjectType = null;
        this.testAnnotator = null;
        this.testOntologyUri1 = null;
        this.differentTestOntologyUri = null;
        this.testTagLiteral = null;
        this.testLongTagLiteral = null;
        
        this.testInvalidAnnotationId = null;
        this.testInvalidAnnotatedObjectType = null;
        this.testInvalidAnnotatedObjectUri = null;
        this.testInvalidOntologyTermUri = null;
        
        this.testOntologyTermLabel = null;
        this.testOntologyDescendant = null;
        this.testOntologyDescendantsChild = null;
    }
    
    /**
     * Test method for {@link com.github.ansell.oas.data.DataHandler#createBlankNode()} .
     */
    @Test
    public void testCreateBlankNode()
    {
        final BNode result = this.dataHandler.createBlankNode();
        
        Assert.assertNotNull(result);
        Assert.assertNotNull(result.getID());
    }
    
    /**
     * Test method for {@link com.github.ansell.oas.data.DataHandler#createURI(java.lang.String)} .
     */
    @Test
    public void testCreateLiteral()
    {
        final Literal plainLiteralTest = this.dataHandler.createLiteral(this.testLiteralString);
        
        Assert.assertEquals(plainLiteralTest.stringValue(), this.testLiteralString);
        Assert.assertNull(plainLiteralTest.getDatatype());
        Assert.assertNull(plainLiteralTest.getLanguage());
        
        final Literal datatypedLiteralTest = this.dataHandler.createLiteral(this.testLiteralString, this.testDatatype);
        
        Assert.assertEquals(datatypedLiteralTest.stringValue(), this.testLiteralString);
        Assert.assertEquals(datatypedLiteralTest.getDatatype(), this.testDatatype);
        Assert.assertNull(plainLiteralTest.getLanguage());
        
        final Literal languageLiteralTest = this.dataHandler.createLiteral(this.testLiteralString, this.testLanguage);
        
        Assert.assertEquals(languageLiteralTest.stringValue(), this.testLiteralString);
        Assert.assertNull(languageLiteralTest.getDatatype());
        // SPARQL specs say that tests on language need to be normalized to lower case first
        Assert.assertEquals(languageLiteralTest.getLanguage().toLowerCase(), this.testLanguage.toLowerCase());
    }
    
    /**
     * Test method for {@link com.github.ansell.oas.data.DataHandler#createURI(java.lang.String)} .
     */
    @Test
    public void testCreateUri()
    {
        final URI result = this.dataHandler.createURI(this.testUriString);
        
        Assert.assertEquals(result.stringValue(), this.testUriString);
    }
    
    /**
     * Tests the data handler by storing and retrieving an object to ensure that the object entered
     * the dataset, before deleting it and then verifying that it was not there anymore.
     * 
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#deleteAnnotation(org.openrdf.model.URI)} .
     */
    @Test
    public void testDeleteAnnotation() throws Exception
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the deletion part
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        Assert.assertTrue(this.dataHandler.deleteAnnotation(this.testUniqueAnnotationId));
        
        // then try to delete an annotation that we didn't create
        Assert.assertFalse(
                "Data Handler reported that it successfully deleted an annotation that should not have existed",
                this.dataHandler.deleteAnnotation(this.testInvalidAnnotationId));
        
        final Annotation testDeletedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        Assert.assertNull(
                "Annotation was not deleted correctly, as the result of a getAnnotation on a deleted annotation should be a null value",
                testDeletedAnnotation);
        
        // try to delete the created and deleted annotation again
        Assert.assertFalse("Data Handler reported that it successfully deleted an annotation twice",
                this.dataHandler.deleteAnnotation(this.testUniqueAnnotationId));
    }
    
    /**
     * Tests that a single valid ontology with no version can be stored using the
     * dataHandler.storeOntology method, without specifying that old versions must be deleted
     */
    @Test
    public void testDeleteOntologyAllVersions()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        Assert.assertTrue(this.dataHandler.deleteOntologyAllVersions(storeOntology.getOntologyUri()));
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertTrue(ontologiesByUri.isEmpty());
    }
    
    /**
     * Tests that a single valid ontology with no version can be stored using the
     * dataHandler.storeOntology method, without specifying that old versions must be deleted
     */
    @Test
    public void testDeleteOntologyVersion()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertTrue(this.dataHandler.deleteOntologyVersion(storeOntology));
        
        final List<Ontology> ontologiesByUri2 = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri2);
        Assert.assertTrue(ontologiesByUri2.isEmpty());
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationsByOntologyTermUri(org.openrdf.model.URI)}
     * .
     */
    @Test
    public void testDeleteOntologyWithMatchingAnnotations()
    {
        final Ontology firstOntology = this.getNewOntology();
        
        firstOntology.setOntologyUri(this.testOntologyUri1);
        firstOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology2-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(firstOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final URI nextUri = this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0000001");
        
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(nextUri);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the search functionality
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        // Now, try and fetch annotations based on the object uri
        final Collection<Annotation> annotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(nextUri);
        
        Assert.assertEquals(1, annotationsByOntologyTermUri.size());
        
        final Annotation searchResult = annotationsByOntologyTermUri.iterator().next();
        
        Assert.assertEquals(this.testAnnotatedObjectUri, searchResult.getAnnotatedObjectUri());
        Assert.assertEquals(this.testAnnotatedObjectType, searchResult.getAnnotatedObjectType());
        Assert.assertEquals(this.testUniqueAnnotationId, searchResult.getAnnotationUniqueId());
        Assert.assertEquals(nextUri, searchResult.getOntologyTermUri());
        
        try
        {
            this.dataHandler.deleteOntologyVersion(storeOntology);
            Assert.fail("Expected an exception when trying to delete an ontology that had annotations referencing a term from inside the ontology");
        }
        catch(final ResourceException re)
        {
            Assert.assertEquals(
                    "Expected HTTP 409 when trying to delete an ontology that had annotations referencing a term from inside the ontology",
                    Status.CLIENT_ERROR_CONFLICT, re.getStatus());
        }
        
    }
    
    @Test
    public void testDownloadOntologyCounts() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        
        // DebugUtils.printStatements(result);
        // DebugUtils.printStatements(result.filter(OWL.THING, null, null));
        // DebugUtils.printStatements(result.filter(null, OasConstants.OAS_URI_RECURSIVE_COUNT,
        // null));
        
        final URI currentParent = OWL.THING;
        
        final Model result =
                this.downloadOntologySubsetToModel(nextOntology, currentParent, this.recursiveFalse,
                        this.childrenCountTrue);
        AbstractTestUtils.assertModel(result, 530, 133, 4, 271);
        
        final int nextExpectedRecursive =
                result.filter(OWL.THING, OasConstants.OAS_URI_RECURSIVE_COUNT, null).objectLiteral().intValue();
        final int nextExpectedDirect =
                result.filter(OWL.THING, OasConstants.OAS_URI_DIRECT_COUNT, null).objectLiteral().intValue();
        Assert.assertEquals(1448, nextExpectedRecursive);
        Assert.assertEquals(132, nextExpectedDirect);
        
        final List<URI> currentPath = new ArrayList<URI>();
        this.checkCounts(nextOntology, currentParent, nextExpectedRecursive, nextExpectedDirect, 0, currentPath);
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntology method
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologyNoVersion() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntology(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType());
        
        // Assert.assertEquals(-1, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 11000000);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 13000000);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        Assert.assertEquals(44335, statementCollector.getStatements().size());
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method with a URI that contains three direct links, and
     * many more recursive links.
     * 
     * 
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetManyMoreLinksWithRecursive() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        // verify that there is only one ontology stored with this URI at this point
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        
        final ByteArrayOutputStream fullOntologyOutputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntology(storeOntology, fullOntologyOutputStream,
                RDFFormat.BINARY.getDefaultMIMEType());
        
        final RDFParser fullParser = Rio.createParser(RDFFormat.BINARY);
        final StatementCollector fullStatementCollector = new StatementCollector();
        fullParser.setRDFHandler(fullStatementCollector);
        fullParser.parse(new ByteArrayInputStream(fullOntologyOutputStream.toByteArray()), "");
        
        Assert.assertEquals(44335, fullStatementCollector.getStatements().size());
        
        final ByteArrayOutputStream inferredOntologyOutputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologyWithInferences(storeOntology, inferredOntologyOutputStream,
                RDFFormat.BINARY.getDefaultMIMEType());
        
        final RDFParser inferredParser = Rio.createParser(RDFFormat.BINARY);
        final StatementCollector inferredStatementCollector = new StatementCollector();
        inferredParser.setRDFHandler(inferredStatementCollector);
        inferredParser.parse(new ByteArrayInputStream(inferredOntologyOutputStream.toByteArray()), "");
        
        Assert.assertEquals(47338, inferredStatementCollector.getStatements().size());
        
        final Repository nextRepository = new SailRepository(new MemoryStore());
        nextRepository.initialize();
        
        final RepositoryConnection nextConn = nextRepository.getConnection();
        
        nextConn.begin();
        nextConn.add(inferredStatementCollector.getStatements());
        nextConn.commit();
        
        Assert.assertEquals(47338, nextConn.size());
        
        final TupleQuery parentQuery =
                nextConn.prepareTupleQuery(
                        QueryLanguage.SPARQL,
                        "SELECT DISTINCT ?parent ?child  WHERE { ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf>+ ?parent . FILTER(isIRI(?child) && isIRI(?parent)) .  } ");
        
        parentQuery.setBinding("parent", this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025131"));
        
        final TupleQueryResult parentQueryResult = parentQuery.evaluate();
        
        int parentResults = 0;
        
        while(parentQueryResult.hasNext())
        {
            parentResults++;
            final BindingSet next = parentQueryResult.next();
            
            // this.log.info("next: {}", next);
        }
        
        Assert.assertEquals(1078, parentResults);
        
        final TupleQuery parentQuery2 =
                nextConn.prepareTupleQuery(
                        QueryLanguage.SPARQL,
                        "SELECT DISTINCT ?parent ?child  WHERE { ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf>+ ?parent . FILTER(isIRI(?child) && isIRI(?parent)) .  } ");
        
        parentQuery2.setBinding("parent", this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0000034"));
        
        final TupleQueryResult parentQuery2Result = parentQuery2.evaluate();
        
        int parentResults2 = 0;
        
        while(parentQuery2Result.hasNext())
        {
            parentResults2++;
            final BindingSet next = parentQuery2Result.next();
            
            // this.log.info("next: {}", next);
        }
        
        Assert.assertEquals(23, parentResults2);
        
        final TupleQuery parentQuery3 =
                nextConn.prepareTupleQuery(
                        QueryLanguage.SPARQL,
                        "SELECT DISTINCT ?parent ?child  WHERE { ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf>+ ?parent . FILTER(isIRI(?child) && isIRI(?parent)) .  } ");
        
        parentQuery3.setBinding("parent", OWL.THING);
        
        final TupleQueryResult parentQuery3Result = parentQuery3.evaluate();
        
        int parentResults3 = 0;
        
        while(parentQuery3Result.hasNext())
        {
            parentResults3++;
            final BindingSet next = parentQuery3Result.next();
            
            // this.log.info("next: {}", next);
        }
        
        Assert.assertEquals(1448, parentResults3);
        
        final TupleQuery childQuery =
                nextConn.prepareTupleQuery(
                        QueryLanguage.SPARQL,
                        "SELECT DISTINCT ?parent ?child  WHERE { ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf>+ ?parent . FILTER(isIRI(?child) && isIRI(?parent)) .  } ");
        
        childQuery.setBinding("child", this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025131"));
        
        final TupleQueryResult childQueryResult = childQuery.evaluate();
        
        int childResults = 0;
        
        while(childQueryResult.hasNext())
        {
            childResults++;
            final BindingSet next = childQueryResult.next();
            
            // this.log.info("next: {}", next);
        }
        
        Assert.assertEquals(1, childResults);
        
        childQuery.clearBindings();
        
        final TupleQuery childQuery2 =
                nextConn.prepareTupleQuery(
                        QueryLanguage.SPARQL,
                        "SELECT DISTINCT ?parent ?child  WHERE { ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf>+ ?parent . FILTER(isIRI(?child) && isIRI(?parent)) .  } ");
        
        childQuery2.setBinding("child", this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0000034"));
        
        final TupleQueryResult childQuery2Result = childQuery2.evaluate();
        
        int childResults2 = 0;
        
        while(childQuery2Result.hasNext())
        {
            childResults2++;
            final BindingSet next = childQuery2Result.next();
            
            // this.log.info("next: {}", next);
        }
        
        Assert.assertEquals(3, childResults2);
        
        final Model fullyInferred = new LinkedHashModel(inferredStatementCollector.getStatements());
        
        final Model filter = fullyInferred.filter(this.testOntologyUri1, null, null);
        
        Assert.assertEquals(15, filter.size());
        
        final Model filter2 =
                fullyInferred.filter(this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025131"), null,
                        null);
        
        Assert.assertEquals(13, filter2.size());
        
        // having verified that the ontology was accurately stored, do the download subset test
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(storeOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025131"), this.recursiveTrue,
                this.childrenCountFalse);
        
        // Assert.assertEquals(-1, outputStream.size());
        // FIXME: Investigate why this is so low when there are many recursive children expected
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 340000);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 370000);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final Model model = new LinkedHashModel();
        parser.setRDFHandler(new StatementCollector(model));
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        // Three links to direct children along with recursive children from this URI
        AbstractTestUtils.assertModel(model, 2156, 1079, 2, 2156);
        
        // having verified that the ontology was accurately stored, do the download subset test
        final ByteArrayOutputStream outputStream2 = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(storeOntology, outputStream2, RDFFormat.RDFJSON.getDefaultMIMEType(),
                OWL.THING, this.recursiveTrue, this.childrenCountFalse);
        
        // Assert.assertEquals(-1, outputStream2.size());
        // FIXME: Investigate why this is so low when there are many recursive children expected
        Assert.assertTrue("Ontology download was smaller than expected", outputStream2.size() > 450000);
        Assert.assertTrue("Ontology download was larger than expected", outputStream2.size() < 500000);
        
        final RDFParser parser2 = Rio.createParser(RDFFormat.RDFJSON);
        
        final Model model2 = new LinkedHashModel();
        parser2.setRDFHandler(new StatementCollector(model2));
        parser2.parse(new ByteArrayInputStream(outputStream2.toByteArray()), "");
        
        // Three links to direct children along with recursive children from this URI
        AbstractTestUtils.assertModel(model2, 2896, 1449, 2, 2895);
        
        // Assert.fail("Should not reach here without failing");
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method with a URI that is directly linked from owl:Thing
     * but does not contain any child links itself.
     * 
     * 
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetNoLinksNoRecursive() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0007107"), this.recursiveFalse,
                this.childrenCountFalse);
        
        Assert.assertEquals(3, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 1);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 4);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        // No links from this URI to anything else
        Assert.assertEquals(0, statementCollector.getStatements().size());
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method with a URI that is directly linked from owl:Thing
     * but does not contain any child links itself.
     * 
     * 
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetNoLinksWithRecursive() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0007107"), this.recursiveTrue,
                this.childrenCountFalse);
        
        // Assert.assertEquals(3, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 1);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 4);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        // No links from this URI to anything else
        Assert.assertEquals(0, statementCollector.getStatements().size());
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method without the parent specified.
     * 
     * In the absence of a specific parent, the subset method must default to using owl:Thing as the
     * parent.
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetNoParentNoRecursive() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                null, this.recursiveFalse, this.childrenCountFalse);
        
        // Assert.assertEquals(-1, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 43000);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 48000);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        Assert.assertEquals(264, statementCollector.getStatements().size());
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method with a URI that is directly linked from owl:Thing
     * but does not contain any child links itself.
     * 
     * 
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetOneLinkNoRecursive() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025005"), this.recursiveFalse,
                this.childrenCountFalse);
        
        // Assert.assertEquals(-1, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 300);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 500);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        // One link from this URI to direct children, with a label
        Assert.assertEquals(2, statementCollector.getStatements().size());
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method with a URI that is directly linked from owl:Thing
     * but does not contain any child links itself.
     * 
     * 
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetOneLinkWithRecursive() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025005"), this.recursiveTrue,
                this.childrenCountFalse);
        
        // Assert.assertEquals(-1, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 4500);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 5500);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final Model model = new LinkedHashModel();
        parser.setRDFHandler(new StatementCollector(model));
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        Assert.assertEquals(30, model.size());
        Assert.assertEquals(16, model.subjects().size());
        Assert.assertEquals(2, model.predicates().size());
        Assert.assertEquals(30, model.objects().size());
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method with the default parent, owl:Thing specified.
     * 
     * 
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetOWLThingParentNoRecursive() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                OWL.THING, this.recursiveFalse, this.childrenCountFalse);
        
        // Assert.assertEquals(-1, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 43000);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 48000);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        Assert.assertEquals(264, statementCollector.getStatements().size());
        
        // System.out.println(outputStream.toString("UTF-8"));
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method with a URI that contains three direct links, and
     * many more recursive links.
     * 
     * 
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetThreeLinksNoRecursive() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025131"), this.recursiveFalse,
                this.childrenCountFalse);
        
        // Assert.assertEquals(-1, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 1000);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 1300);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        
        final StatementCollector statementCollector = new StatementCollector();
        parser.setRDFHandler(statementCollector);
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        // Three links from this URI to direct children plus labels
        Assert.assertEquals(6, statementCollector.getStatements().size());
    }
    
    /**
     * Tests that a single valid ontology with no version can be downloaded using the
     * dataHandler.downloadOntologySubset method with a URI that contains three direct links, and
     * many more recursive links.
     * 
     * 
     * 
     * @throws Exception
     */
    @Test
    public void testDownloadOntologySubsetThreeLinksNoRecursiveWithChildrenCount() throws Exception
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
        
        Assert.assertEquals(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, ontologiesByUri.get(0).getOntologyVersionUri());
        final ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        
        this.dataHandler.downloadOntologySubset(nextOntology, outputStream, RDFFormat.RDFJSON.getDefaultMIMEType(),
                this.dataHandler.createURI("http://purl.obolibrary.org/obo/PO_0025131"), this.recursiveFalse,
                this.childrenCountTrue);
        
        // Assert.assertEquals(-1, outputStream.size());
        Assert.assertTrue("Ontology download was smaller than expected", outputStream.size() > 2000);
        Assert.assertTrue("Ontology download was larger than expected", outputStream.size() < 2600);
        
        final RDFParser parser = Rio.createParser(RDFFormat.RDFJSON);
        final Model model = new LinkedHashModel();
        final StatementCollector statementCollector = new StatementCollector(model);
        parser.setRDFHandler(statementCollector);
        parser.parse(new ByteArrayInputStream(outputStream.toByteArray()), "");
        
        // this.log.info("subjects: {}", model.subjects());
        // this.log.info("model: {}", model);
        // this.log.info("objects: {}", model.objects());
        
        // Three links from this URI to direct children plus labels plus children counts
        AbstractTestUtils.assertModel(model, 14, 4, 4, 13);
    }
    
    @Test
    public void testGetAllOntologiesEmpty()
    {
        final List<Ontology> ontologies = this.dataHandler.getOntologies(0, 100);
        
        Assert.assertNotNull(ontologies);
        
        Assert.assertTrue(ontologies.isEmpty());
    }
    
    @Test
    public void testGetAllOntologiesSingle()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologies = this.dataHandler.getOntologies(0, 100);
        
        Assert.assertNotNull(ontologies);
        
        Assert.assertEquals(1, ontologies.size());
        final Ontology resultOntology = ontologies.get(0);
        
        Assert.assertEquals(this.testOntologyUri1, resultOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, resultOntology.getOntologyVersionUri());
    }
    
    @Test
    public void testGetAllOntologiesTwo()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final Ontology secondOntology = this.getNewOntology();
        
        secondOntology.setOntologyUri(this.testOntologyUri2);
        secondOntology.setOntologyVersionUri(this.testOntologyVersionUri2);
        
        final InputStream secondInputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", secondInputStream);
        
        final Ontology secondStoreOntology =
                this.dataHandler.storeOntology(secondOntology, secondInputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(secondStoreOntology);
        Assert.assertNotNull(secondStoreOntology.getOntologyUri());
        Assert.assertNotNull(secondStoreOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri2, secondStoreOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri2, secondStoreOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyVersionUri2, secondStoreOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologies = this.dataHandler.getOntologies(0, 100);
        
        Assert.assertNotNull(ontologies);
        
        Assert.assertEquals(2, ontologies.size());
        
        Ontology resultOntology1 = null;
        Ontology resultOntology2 = null;
        
        for(final Ontology nextResultOntology : ontologies)
        {
            Assert.assertNotNull(nextResultOntology);
            
            if(nextResultOntology.getOntologyUri().equals(this.testOntologyUri1))
            {
                Assert.assertNull(resultOntology1);
                resultOntology1 = nextResultOntology;
            }
            else if(nextResultOntology.getOntologyUri().equals(this.testOntologyUri2))
            {
                Assert.assertNull(resultOntology2);
                resultOntology2 = nextResultOntology;
            }
            else
            {
                Assert.fail("Found unrecognised ontology URI=" + nextResultOntology.getOntologyUri());
            }
        }
        
        Assert.assertNotNull(resultOntology1);
        Assert.assertNotNull(resultOntology2);
        
        Assert.assertEquals(this.testOntologyUri1, resultOntology1.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, resultOntology1.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri2, resultOntology2.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri2, resultOntology2.getOntologyVersionUri());
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotation(org.openrdf.model.URI)} .
     */
    @Test
    public void testGetAnnotation()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the deletion part
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        final Annotation fakeAnnotation = this.dataHandler.getAnnotation(this.testInvalidAnnotationId);
        
        Assert.assertNull("Get Annotation returned an annotation that should not have existed", fakeAnnotation);
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationCountsByOntologyTermUri(URI, Ontology, boolean)}
     * .
     */
    @Test
    public void testGetAnnotationCountsByOntologyTermUriWithOntologyNonRecursive()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        nextOntology.setInferredOntologyUri(this.testInferredOntologyUri1);
        
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyDescendant);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the search functionality
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        // Now, try and fetch annotations based on the object uri
        final Collection<Annotation> annotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testOntologyDescendant);
        
        Assert.assertEquals(1, annotationsByOntologyTermUri.size());
        
        // Should not find the annotation without loading the ontology in
        final ResourceCount resourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testOntologyDescendant, nextOntology,
                        this.recursiveFalse);
        Assert.assertEquals(0, resourceCount.getCount());
        
        // Now actually load the ontology to see if it finds it now
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        // Now we should be able to find the annotation
        final ResourceCount afterLoadResourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testOntologyDescendant, storeOntology,
                        this.recursiveFalse);
        Assert.assertEquals(1, afterLoadResourceCount.getCount());
        
        final Annotation searchResult = annotationsByOntologyTermUri.iterator().next();
        
        Assert.assertEquals(this.testAnnotatedObjectUri, searchResult.getAnnotatedObjectUri());
        Assert.assertEquals(this.testAnnotatedObjectType, searchResult.getAnnotatedObjectType());
        Assert.assertEquals(this.testUniqueAnnotationId, searchResult.getAnnotationUniqueId());
        Assert.assertEquals(this.testOntologyDescendant, searchResult.getOntologyTermUri());
        
        final Collection<Annotation> invalidAnnotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testInvalidOntologyTermUri);
        
        Assert.assertEquals(0, invalidAnnotationsByOntologyTermUri.size());
        
        final ResourceCount invalidResourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testInvalidOntologyTermUri, storeOntology,
                        this.recursiveFalse);
        
        Assert.assertEquals(0, invalidResourceCount.getCount());
        
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationCountsByOntologyTermUri(URI, Ontology, boolean)}
     * .
     */
    @Test
    public void testGetAnnotationCountsByOntologyTermUriWithOntologyRecursive()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        nextOntology.setInferredOntologyUri(this.testInferredOntologyUri1);
        
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyDescendant);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the search functionality
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        // Now, try and fetch annotations based on the object uri
        final Collection<Annotation> annotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testOntologyDescendant);
        
        Assert.assertEquals(1, annotationsByOntologyTermUri.size());
        
        final Annotation nextChildAnnotation = this.getNewAnnotation();
        
        nextChildAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId2);
        nextChildAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextChildAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextChildAnnotation.setAnnotator(this.testAnnotator);
        nextChildAnnotation.setOntologyTermUri(this.testOntologyDescendantsChild);
        nextChildAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextChildAnnotation));
        
        // Should not find the annotation without loading the ontology in
        final ResourceCount resourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testOntologyDescendant, nextOntology,
                        this.recursiveTrue);
        Assert.assertEquals(0, resourceCount.getCount());
        
        // Now actually load the ontology to see if it finds it now
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        // Now we should be able to find the annotation
        final ResourceCount afterLoadResourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testOntologyDescendant, storeOntology,
                        this.recursiveTrue);
        Assert.assertEquals(2, afterLoadResourceCount.getCount());
        
        final Annotation searchResult = annotationsByOntologyTermUri.iterator().next();
        
        Assert.assertEquals(this.testAnnotatedObjectUri, searchResult.getAnnotatedObjectUri());
        Assert.assertEquals(this.testAnnotatedObjectType, searchResult.getAnnotatedObjectType());
        Assert.assertEquals(this.testUniqueAnnotationId, searchResult.getAnnotationUniqueId());
        Assert.assertEquals(this.testOntologyDescendant, searchResult.getOntologyTermUri());
        
        final Collection<Annotation> invalidAnnotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testInvalidOntologyTermUri);
        
        Assert.assertEquals(0, invalidAnnotationsByOntologyTermUri.size());
        
        final ResourceCount invalidResourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testInvalidOntologyTermUri, storeOntology,
                        this.recursiveTrue);
        
        Assert.assertEquals(0, invalidResourceCount.getCount());
        
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationCountsByOntologyTermUri(URI, Ontology, boolean)}
     * .
     */
    @Test
    public void testGetAnnotationCountsByOntologyTermUriWithoutOntologyNonRecursive()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the search functionality
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        // Now, try and fetch annotations based on the object uri
        final Collection<Annotation> annotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testOntologyUri1);
        
        Assert.assertEquals(1, annotationsByOntologyTermUri.size());
        
        final ResourceCount resourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testOntologyUri1, null, this.recursiveFalse);
        
        Assert.assertEquals(1, resourceCount.getCount());
        
        final Annotation searchResult = annotationsByOntologyTermUri.iterator().next();
        
        Assert.assertEquals(this.testAnnotatedObjectUri, searchResult.getAnnotatedObjectUri());
        Assert.assertEquals(this.testAnnotatedObjectType, searchResult.getAnnotatedObjectType());
        Assert.assertEquals(this.testUniqueAnnotationId, searchResult.getAnnotationUniqueId());
        Assert.assertEquals(this.testOntologyUri1, searchResult.getOntologyTermUri());
        
        final Collection<Annotation> invalidAnnotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testInvalidOntologyTermUri);
        
        Assert.assertEquals(0, invalidAnnotationsByOntologyTermUri.size());
        
        final ResourceCount invalidResourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testInvalidOntologyTermUri, null,
                        this.recursiveFalse);
        
        Assert.assertEquals(0, invalidResourceCount.getCount());
        
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationCountsByOntologyTermUri(URI, Ontology, boolean)}
     * .
     */
    @Test
    public void testGetAnnotationCountsByOntologyTermUriWithoutOntologyRecursive()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the search functionality
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        // Now, try and fetch annotations based on the object uri
        final Collection<Annotation> annotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testOntologyUri1);
        
        Assert.assertEquals(1, annotationsByOntologyTermUri.size());
        
        final Annotation nextChildAnnotation = this.getNewAnnotation();
        
        nextChildAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextChildAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextChildAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextChildAnnotation.setAnnotator(this.testAnnotator);
        nextChildAnnotation.setOntologyTermUri(this.testOntologyDescendantsChild);
        nextChildAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextChildAnnotation));
        
        final ResourceCount resourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testOntologyUri1, null, this.recursiveTrue);
        
        // Recursion does not count if there is no ontology, so only the direct annotation will be
        // returned, as there must be an ontology to define the recursion tree
        Assert.assertEquals(1, resourceCount.getCount());
        
        final Annotation searchResult = annotationsByOntologyTermUri.iterator().next();
        
        Assert.assertEquals(this.testAnnotatedObjectUri, searchResult.getAnnotatedObjectUri());
        Assert.assertEquals(this.testAnnotatedObjectType, searchResult.getAnnotatedObjectType());
        Assert.assertEquals(this.testUniqueAnnotationId, searchResult.getAnnotationUniqueId());
        Assert.assertEquals(this.testOntologyUri1, searchResult.getOntologyTermUri());
        
        final Collection<Annotation> invalidAnnotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testInvalidOntologyTermUri);
        
        Assert.assertEquals(0, invalidAnnotationsByOntologyTermUri.size());
        
        final ResourceCount invalidResourceCount =
                this.dataHandler.getAnnotationCountsByOntologyTermUri(this.testInvalidOntologyTermUri, null, false);
        
        Assert.assertEquals(0, invalidResourceCount.getCount());
        
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationsByAnnotatedObjectType(org.openrdf.model.URI)}
     * .
     */
    @Test
    public void testGetAnnotationsByAnnotatedObjectType()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the search functionality
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        // Now, try and fetch annotations based on the object type
        final Collection<Annotation> annotationsByObjectType =
                this.dataHandler.getAnnotationsByAnnotatedObjectType(this.testAnnotatedObjectType);
        
        Assert.assertEquals(1, annotationsByObjectType.size());
        
        final Annotation searchResult = annotationsByObjectType.iterator().next();
        
        Assert.assertEquals(this.testAnnotatedObjectType, searchResult.getAnnotatedObjectType());
        Assert.assertEquals(this.testAnnotatedObjectUri, searchResult.getAnnotatedObjectUri());
        Assert.assertEquals(this.testUniqueAnnotationId, searchResult.getAnnotationUniqueId());
        
        final Collection<Annotation> invalidAnnotationsByObjectType =
                this.dataHandler.getAnnotationsByAnnotatedObjectType(this.testInvalidAnnotatedObjectType);
        
        Assert.assertEquals(0, invalidAnnotationsByObjectType.size());
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationsByAnnotatedObjectUri(org.openrdf.model.URI)}
     * .
     */
    @Test
    public void testGetAnnotationsByAnnotatedObjectUri()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the search functionality
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        // Now, try and fetch annotations based on the object uri
        final Collection<Annotation> annotationsByObjectUri =
                this.dataHandler.getAnnotationsByAnnotatedObjectUri(this.testAnnotatedObjectUri);
        
        Assert.assertEquals(1, annotationsByObjectUri.size());
        
        final Annotation searchResult = annotationsByObjectUri.iterator().next();
        
        Assert.assertEquals(this.testAnnotatedObjectUri, searchResult.getAnnotatedObjectUri());
        Assert.assertEquals(this.testAnnotatedObjectType, searchResult.getAnnotatedObjectType());
        Assert.assertEquals(this.testUniqueAnnotationId, searchResult.getAnnotationUniqueId());
        
        final Collection<Annotation> invalidAnnotationsByObjectUri =
                this.dataHandler.getAnnotationsByAnnotatedObjectUri(this.testInvalidAnnotatedObjectUri);
        
        Assert.assertEquals(0, invalidAnnotationsByObjectUri.size());
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationsByOntologyTermUri(org.openrdf.model.URI)}
     * .
     */
    @Test
    public void testGetAnnotationsByOntologyTermUri()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the search functionality
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        // Now, try and fetch annotations based on the object uri
        final Collection<Annotation> annotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testOntologyUri1);
        
        Assert.assertEquals(1, annotationsByOntologyTermUri.size());
        
        final Annotation searchResult = annotationsByOntologyTermUri.iterator().next();
        
        Assert.assertEquals(this.testAnnotatedObjectUri, searchResult.getAnnotatedObjectUri());
        Assert.assertEquals(this.testAnnotatedObjectType, searchResult.getAnnotatedObjectType());
        Assert.assertEquals(this.testUniqueAnnotationId, searchResult.getAnnotationUniqueId());
        Assert.assertEquals(this.testOntologyUri1, searchResult.getOntologyTermUri());
        
        final Collection<Annotation> invalidAnnotationsByOntologyTermUri =
                this.dataHandler.getAnnotationsByOntologyTermUri(this.testInvalidOntologyTermUri);
        
        Assert.assertEquals(0, invalidAnnotationsByOntologyTermUri.size());
    }
    
    @Test
    public void testGetChildrenCountNonRecursiveDescendant()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        nextOntology.setInferredOntologyUri(this.testInferredOntologyUri1);
        // nextOntology
        // .setDirectChildQuery(" ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf> ?parent . FILTER(isIRI(?child) && isIRI(?parent)) . ");
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        final ConcurrentMap<URI, ResourceCount> childrenCount =
                this.dataHandler.getChildrenCount(storeOntology, Arrays.asList(this.testOntologyDescendant),
                        this.recursiveFalse);
        
        Assert.assertTrue(childrenCount.containsKey(this.testOntologyDescendant));
        Assert.assertEquals(3, childrenCount.get(this.testOntologyDescendant).getCount());
    }
    
    @Test
    public void testGetChildrenCountNonRecursiveOwlThing()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        nextOntology.setInferredOntologyUri(this.testInferredOntologyUri1);
        // nextOntology
        // .setDirectChildQuery(" ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf> ?parent . FILTER(isIRI(?child) && isIRI(?parent)) . ");
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        final ConcurrentMap<URI, ResourceCount> childrenCount =
                this.dataHandler.getChildrenCount(storeOntology, Arrays.asList(OasConstants.OWL_THING),
                        this.recursiveFalse);
        
        Assert.assertTrue(childrenCount.containsKey(OasConstants.OWL_THING));
        Assert.assertEquals(132, childrenCount.get(OasConstants.OWL_THING).getCount());
    }
    
    @Test
    public void testGetChildrenCountRecursiveDescendant()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        nextOntology.setInferredOntologyUri(this.testInferredOntologyUri1);
        // nextOntology
        // .setRecursiveChildQuery(" ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf>+ ?parent . FILTER(isIRI(?child) && isIRI(?parent)) . ");
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        final ConcurrentMap<URI, ResourceCount> childrenCount =
                this.dataHandler.getChildrenCount(storeOntology, Arrays.asList(this.testOntologyDescendant),
                        this.recursiveTrue);
        
        Assert.assertTrue(childrenCount.containsKey(this.testOntologyDescendant));
        Assert.assertEquals(1078, childrenCount.get(this.testOntologyDescendant).getCount());
    }
    
    @Test
    public void testGetChildrenCountRecursiveOwlThing()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        nextOntology.setInferredOntologyUri(this.testInferredOntologyUri1);
        // nextOntology
        // .setRecursiveChildQuery(" ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf>+ ?parent . FILTER(isIRI(?child) && isIRI(?parent)) . ");
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        final ConcurrentMap<URI, ResourceCount> childrenCount =
                this.dataHandler.getChildrenCount(storeOntology, Arrays.asList(OasConstants.OWL_THING),
                        this.recursiveTrue);
        
        Assert.assertTrue(childrenCount.containsKey(OasConstants.OWL_THING));
        Assert.assertEquals(1448, childrenCount.get(OasConstants.OWL_THING).getCount());
    }
    
    @Test
    public void testGetChildrenNonRecursiveOwlThing()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        nextOntology.setInferredOntologyUri(this.testInferredOntologyUri1);
        nextOntology
                .setDirectChildQuery(" ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf> ?parent . FILTER(isIRI(?child) && isIRI(?parent)) . ");
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        final Map<URI, Collection<URI>> children =
                this.dataHandler.getChildren(storeOntology, OasConstants.OWL_THING, this.recursiveFalse, 10000, 0);
        
        Assert.assertNotNull(children);
        
        Assert.assertEquals(1, children.size());
        
        Assert.assertTrue(children.containsKey(OasConstants.OWL_THING));
        
        Assert.assertEquals(132, children.get(OasConstants.OWL_THING).size());
    }
    
    @Test
    public void testGetChildrenRecursiveOwlThing()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        nextOntology.setInferredOntologyUri(this.testInferredOntologyUri1);
        nextOntology
                .setRecursiveChildQuery(" ?child a <http://www.w3.org/2002/07/owl#Class> . ?child <http://www.w3.org/2000/01/rdf-schema#subClassOf>+ ?parent . FILTER(isIRI(?child) && isIRI(?parent)) . ");
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        final Map<URI, Collection<URI>> children =
                this.dataHandler.getChildren(storeOntology, OasConstants.OWL_THING, this.recursiveTrue, 10000, 0);
        
        Assert.assertNotNull(children);
        
        Assert.assertEquals(1, children.size());
        
        Assert.assertTrue(children.containsKey(OasConstants.OWL_THING));
        
        Assert.assertEquals(1448, children.get(OasConstants.OWL_THING).size());
    }
    
    @Test
    public void testGetLabelsJustVersionGraph()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        // must always setup ontology version URI to be able to use getLabels
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        Assert.assertNotNull(storeOntology.getInferredOntologyUri());
        
        final List<Literal> labelsForUri = this.dataHandler.getLabelsForUri(storeOntology, this.testOntologyTermLabel);
        
        Assert.assertNotNull(labelsForUri);
        Assert.assertFalse(labelsForUri.isEmpty());
        Assert.assertEquals("Found more than one label : " + labelsForUri.toString(), 1, labelsForUri.size());
    }
    
    @Test
    public void testGetLabelsWithInferredGraph()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        // must always setup ontology version URI to be able to use getLabels
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        Assert.assertNotNull(storeOntology.getInferredOntologyUri());
        
        final List<Literal> labelsForUri = this.dataHandler.getLabelsForUri(storeOntology, this.testOntologyTermLabel);
        
        Assert.assertNotNull(labelsForUri);
        Assert.assertFalse(labelsForUri.isEmpty());
        Assert.assertEquals("Found more than one label : " + labelsForUri.toString(), 1, labelsForUri.size());
    }
    
    @Test
    public void testGetLogDetailsEmptyHighOffsetZeroLimit()
    {
        final Model details = this.dataHandler.getLogDetails(100, 0);
        
        Assert.assertTrue(details.isEmpty());
    }
    
    @Test
    public void testGetLogDetailsEmptyZeroOffsetAndHighLimit()
    {
        final Model details = this.dataHandler.getLogDetails(0, 100);
        
        Assert.assertTrue(details.isEmpty());
    }
    
    @Test
    public void testGetLogDetailsEmptyZeroOffsetAndLimit()
    {
        final Model details = this.dataHandler.getLogDetails(0, 0);
        
        Assert.assertTrue(details.isEmpty());
    }
    
    @Test
    public void testGetLogDetailsSingleZeroOffsetAndHighLimit() throws Exception
    {
        this.dataHandler.addLogDetailsForRequest("test message", "urn:test:uri", BaseOasActions.AUDITLOG_GET, new User(
                "anotherUserIdentifier"), "127.0.0.2");
        
        final Model details = this.dataHandler.getLogDetails(0, 100);
        
        Assert.assertFalse(details.isEmpty());
        
        Assert.assertEquals(8, details.size());
        
        final URI subjectURI = GraphUtil.getUniqueSubjectURI(details, null, null);
        Assert.assertEquals(OasConstants.URN_TAG_AUDITLOG_RECORD,
                GraphUtil.getUniqueObjectURI(details, subjectURI, RDF.TYPE));
        Assert.assertEquals("test message", GraphUtil.getUniqueObjectLiteral(details, subjectURI, RDFS.COMMENT)
                .stringValue());
        // Date is variable, so just testing that it is present and unique
        GraphUtil.getUniqueObjectLiteral(details, subjectURI, OasConstants.DC_TERMS_MODIFIED);
        
        Assert.assertEquals(BaseOasActions.AUDITLOG_GET.toString().toString(),
                GraphUtil.getUniqueObjectLiteral(details, subjectURI, OasConstants.URN_TAG_LOGACTION).stringValue());
        Assert.assertEquals(BaseOasActions.AUDITLOG_GET.getMethod().getUri(),
                GraphUtil.getUniqueObjectURI(details, subjectURI, OasConstants.URN_TAG_LOGOPERATION_VERB));
        
        Assert.assertEquals("anotherUserIdentifier",
                GraphUtil.getUniqueObjectLiteral(details, subjectURI, OasConstants.FOAF_MAKER).stringValue());
        
        Assert.assertEquals("urn:test:uri",
                GraphUtil.getUniqueObjectLiteral(details, subjectURI, OasConstants.URN_TAG_AUDITLOG_REF).stringValue());
        
        Assert.assertEquals("127.0.0.2",
                GraphUtil.getUniqueObjectLiteral(details, subjectURI, OasConstants.URN_TAG_USERSIPADDRESS)
                        .stringValue());
    }
    
    @Test
    public void testGetManagedOntologyAll()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final Ontology managedOntology =
                this.dataHandler.getVersionedOntology(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        Assert.assertNotNull(managedOntology);
        Assert.assertNotNull(managedOntology.getOntologyUri());
        Assert.assertNotNull(managedOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, managedOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, managedOntology.getOntologyVersionUri());
        
    }
    
    @Test
    public void testGetManagedOntologyOnlyOntologyUri()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        
        final Ontology managedOntology =
                this.dataHandler.getVersionedOntology(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        Assert.assertNotNull(managedOntology);
        Assert.assertNotNull(managedOntology.getOntologyUri());
        Assert.assertNotNull(managedOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, managedOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyUri1, managedOntology.getOntologyUri());
        
    }
    
    @Test
    public void testGetOntologiesByUriMultipleAll()
    {
        final Ontology firstOntology = this.getNewOntology();
        
        firstOntology.setOntologyUri(this.testOntologyUri1);
        firstOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(firstOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final Ontology secondOntology = this.getNewOntology();
        
        secondOntology.setOntologyUri(this.testOntologyUri1);
        secondOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream2 = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream2);
        
        // store the same ontology again with the same details and verify that now there are 2
        // ontologies returned
        final Ontology storeOntology2 =
                this.dataHandler.storeOntology(secondOntology, inputStream2,
                        "http://test.base.example.org/test/base/uri/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology2);
        Assert.assertNotNull(storeOntology2.getOntologyUri());
        Assert.assertNotNull(storeOntology2.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology2.getOntologyUri());
        Assert.assertNotSame(this.testOntologyVersionUri1, storeOntology2.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(this.testOntologyUri1);
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertEquals(2, ontologiesByUri.size());
        
        final Iterator<Ontology> iterator = ontologiesByUri.iterator();
        
        Assert.assertTrue(iterator.hasNext());
        
        final Ontology firstFetchedOntology = iterator.next();
        
        Assert.assertNotNull(firstFetchedOntology);
        Assert.assertEquals(this.testOntologyUri1, firstFetchedOntology.getOntologyUri());
        Assert.assertNotNull(firstFetchedOntology.getOntologyVersionUri());
        Assert.assertEquals(this.testOntologyVersionUri1, firstFetchedOntology.getOntologyVersionUri());
        
        Assert.assertTrue(iterator.hasNext());
        
        final Ontology secondFetchedOntology = iterator.next();
        
        Assert.assertNotNull(secondFetchedOntology);
        Assert.assertEquals(this.testOntologyUri1, secondFetchedOntology.getOntologyUri());
        Assert.assertNotNull(secondFetchedOntology.getOntologyVersionUri());
        // assertEquals(this.testOntologyVersionUri1,
        // secondFetchedOntology.getOntologyVersionUri());
        
    }
    
    @Test
    public void testGetOntologiesByUriOnlyMultipleOntologyUri()
    {
        final Ontology firstOntology = this.getNewOntology();
        
        firstOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(firstOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        
        final Ontology secondOntology = this.getNewOntology();
        
        secondOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream2 = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream2);
        
        // store the same ontology again with the same details and verify that now there are 2
        // ontologies returned
        final Ontology storeOntology2 =
                this.dataHandler.storeOntology(secondOntology, inputStream2,
                        "http://test.base.example.org/test/base/uri/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology2);
        Assert.assertNotNull(storeOntology2.getOntologyUri());
        Assert.assertNotNull(storeOntology2.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology2.getOntologyUri());
        // ontology version URI in the metadata should have been bumped automatically
        Assert.assertNotSame(this.testOntologyVersionUri1, storeOntology2.getOntologyVersionUri());
        
        // NOTE: The order of this list is reliant on an assumption that the automatically bumped
        // versionURI sorts after the original version URI
        // If this is not the case this test will fail when it expects the first Ontology on the
        // iterator to match the first ontology and the second to match the second
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(this.testOntologyUri1);
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertEquals(2, ontologiesByUri.size());
        
        final Iterator<Ontology> iterator = ontologiesByUri.iterator();
        
        Assert.assertTrue(iterator.hasNext());
        
        final Ontology firstFetchedOntology = iterator.next();
        
        Assert.assertNotNull(firstFetchedOntology);
        Assert.assertEquals(this.testOntologyUri1, firstFetchedOntology.getOntologyUri());
        Assert.assertNotNull(firstFetchedOntology.getOntologyVersionUri());
        
        Assert.assertTrue(iterator.hasNext());
        
        final Ontology secondFetchedOntology = iterator.next();
        
        Assert.assertNotNull(secondFetchedOntology);
        Assert.assertEquals(this.testOntologyUri1, secondFetchedOntology.getOntologyUri());
        Assert.assertNotNull(secondFetchedOntology.getOntologyVersionUri());
        
        Assert.assertNotSame(secondFetchedOntology.getOntologyVersionUri(),
                firstFetchedOntology.getOntologyVersionUri());
    }
    
    @Test
    public void testGetOntologiesByUriOnlySingleOntologyUri()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(this.testOntologyUri1);
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertEquals(1, ontologiesByUri.size());
        
        final Ontology firstOntology = ontologiesByUri.iterator().next();
        
        Assert.assertNotNull(firstOntology);
        Assert.assertEquals(this.testOntologyUri1, firstOntology.getOntologyUri());
    }
    
    @Test
    public void testGetOntologiesByUriSingleAll()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(this.testOntologyUri1);
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertEquals(1, ontologiesByUri.size());
        
        final Ontology firstOntology = ontologiesByUri.iterator().next();
        
        Assert.assertNotNull(firstOntology);
        Assert.assertEquals(this.testOntologyUri1, firstOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, firstOntology.getOntologyVersionUri());
    }
    
    /**
     * Test method for {@link com.github.ansell.oas.data.DataHandler#listAnnotations(int, int)} .
     */
    @Test
    public void testListAnnotations()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Collection<Annotation> annotations = this.dataHandler.listAnnotations(10, 0);
        
        Assert.assertEquals(1, annotations.size());
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#getAnnotationsByOntologyTermUri(org.openrdf.model.URI)}
     * .
     */
    @Test
    public void testSearchAllOntologies()
    {
        final Ontology firstOntology = this.getNewOntology();
        
        firstOntology.setOntologyUri(this.testOntologyUri1);
        firstOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology2-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(firstOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final List<SearchResult> searchAllOntologies = this.dataHandler.searchAllOntologies("embryo", 100, 0);
        
        Assert.assertEquals(1, searchAllOntologies.size());
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#storeAnnotation(com.github.ansell.oas.objects.Annotation)}
     * .
     */
    @Test
    public void testSearchAnnotations()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testLongTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the deletion part
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        final Collection<Annotation> searchAnnotations = this.dataHandler.searchAnnotations("know", 100, 0);
        
        Assert.assertEquals(1, searchAnnotations.size());
    }
    
    @Test
    public void testSearchOntologiesLabels()
    {
        final Ontology firstOntology = this.getNewOntology();
        
        firstOntology.setOntologyUri(this.testOntologyUri1);
        firstOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology2-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(firstOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final List<SearchResult> searchAllOntologies = this.dataHandler.searchOntologyLabels("test", 100, 0);
        
        Assert.assertEquals(1, searchAllOntologies.size());
    }
    
    @Test
    public void testSearchOntologiesLabelsEmbryo()
    {
        final Ontology firstOntology = this.getNewOntology();
        
        firstOntology.setOntologyUri(this.testOntologyUri1);
        firstOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology2-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(firstOntology, inputStream,
                        "http://test.base.example.org/test/base/uri/1/1/", "application/rdf+xml", false);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        final List<SearchResult> searchAllOntologies = this.dataHandler.searchOntologyLabels("embryo", 100, 0);
        
        Assert.assertEquals(0, searchAllOntologies.size());
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#storeAnnotation(com.github.ansell.oas.objects.Annotation)}
     * .
     */
    @Test
    public void testStoreAnnotation()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the deletion part
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
    }
    
    /**
     * Tests that a single null ontology cannot be stored using the dataHandler.storeOntology
     * method, without specifying that old versions must be deleted.
     */
    @Test
    public void testStoreOntologyAllNull()
    {
        final Ontology nextOntology = null;
        
        try
        {
            this.dataHandler.storeOntology(nextOntology, (InputStream)null, null, null, this.deleteOldVersionsFalse);
            
            Assert.fail("Did not find expected IllegalArgumentException");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("must not be null"));
        }
    }
    
    /**
     * Tests that multiple valid ontologies, with the same Ontology IRIs, but different Version IRIs
     * can be stored using the dataHandler.storeOntology method, without specifying that old
     * versions must be deleted.
     */
    @Test
    public void testStoreOntologyMultipleValidDeleteFalse()
    {
        for(int count = 1; count < 10; count++)
        {
            final Ontology nextOntology = this.getNewOntology();
            
            final URI nextOntologyVersionIri = this.dataHandler.createURI(this.testOntologyVersionUriBase + count);
            nextOntology.setOntologyUri(this.testOntologyUri1);
            nextOntology.setOntologyVersionUri(nextOntologyVersionIri);
            
            final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
            
            Assert.assertNotNull("Could not find test file", inputStream);
            
            final Ontology storeOntology =
                    this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                            "application/rdf+xml", this.deleteOldVersionsFalse);
            
            Assert.assertNotNull(storeOntology);
            Assert.assertNotNull(storeOntology.getOntologyUri());
            Assert.assertNotNull(storeOntology.getOntologyVersionUri());
            
            Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
            Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
            
            Assert.assertEquals(nextOntologyVersionIri, storeOntology.getOntologyVersionUri());
            
            // verify that the ontology was stored correctly
            final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
            
            Assert.assertNotNull(ontologiesByUri);
            Assert.assertFalse(ontologiesByUri.isEmpty());
            Assert.assertEquals(count, ontologiesByUri.size());
        }
    }
    
    /**
     * Tests that multiple valid ontologies, with a different Ontology IRI to the supplied Ontology,
     * and different Version IRIs can be stored using the dataHandler.storeOntology method, without
     * specifying that old versions must be deleted.
     */
    @Test
    public void testStoreOntologyMultipleValidDeleteFalseMediumSize()
    {
        for(int count = 1; count < 10; count++)
        {
            final Ontology nextOntology = this.getNewOntology();
            
            final URI nextOntologyVersionIri = this.dataHandler.createURI(this.testOntologyVersionUriBase + count);
            nextOntology.setOntologyUri(this.testOntologyUri1);
            nextOntology.setOntologyVersionUri(nextOntologyVersionIri);
            
            final InputStream inputStream = this.getClass().getResourceAsStream("/ontologies/plant_ontology-v16.owl");
            
            Assert.assertNotNull("Could not find test file", inputStream);
            
            final Ontology storeOntology =
                    this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                            "application/rdf+xml", this.deleteOldVersionsFalse);
            
            Assert.assertNotNull(storeOntology);
            Assert.assertNotNull(storeOntology.getOntologyUri());
            Assert.assertNotNull(storeOntology.getOntologyVersionUri());
            
            Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
            Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
            
            Assert.assertEquals(nextOntologyVersionIri, storeOntology.getOntologyVersionUri());
            
            // verify that the ontology was stored correctly
            final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
            
            Assert.assertNotNull(ontologiesByUri);
            Assert.assertFalse(ontologiesByUri.isEmpty());
            Assert.assertEquals(count, ontologiesByUri.size());
        }
    }
    
    /**
     * Tests that multiple valid ontologies, with the same Ontology IRIs, but different Version IRIs
     * can be stored using the dataHandler.storeOntology method, without specifying that old
     * versions must be deleted.
     */
    @Test
    public void testStoreOntologyMultipleValidDeleteTrue()
    {
        for(int count = 1; count < 10; count++)
        {
            final Ontology nextOntology = this.getNewOntology();
            
            final URI nextOntologyVersionIri = this.dataHandler.createURI(this.testOntologyVersionUriBase + count);
            nextOntology.setOntologyUri(this.testOntologyUri1);
            nextOntology.setOntologyVersionUri(nextOntologyVersionIri);
            
            final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
            
            Assert.assertNotNull("Could not find test file", inputStream);
            
            final Ontology storeOntology =
                    this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                            "application/rdf+xml", this.deleteOldVersionsTrue);
            
            Assert.assertNotNull(storeOntology);
            Assert.assertNotNull(storeOntology.getOntologyUri());
            Assert.assertNotNull(storeOntology.getOntologyVersionUri());
            
            Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
            Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
            
            Assert.assertEquals(nextOntologyVersionIri, storeOntology.getOntologyVersionUri());
            
            // verify that the ontology was stored correctly
            final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
            
            Assert.assertNotNull(ontologiesByUri);
            Assert.assertFalse(ontologiesByUri.isEmpty());
            Assert.assertEquals(1, ontologiesByUri.size());
        }
    }
    
    /**
     * Tests that multiple valid ontologies, with a different Ontology IRI to the supplied Ontology,
     * and different Version IRIs can be stored using the dataHandler.storeOntology method, without
     * specifying that old versions must be deleted.
     */
    @Test
    public void testStoreOntologyMultipleValidDeleteTrueMediumSize()
    {
        for(int count = 1; count < 10; count++)
        {
            final Ontology nextOntology = this.getNewOntology();
            
            final URI nextOntologyVersionIri = this.dataHandler.createURI(this.testOntologyVersionUriBase + count);
            nextOntology.setOntologyUri(this.testOntologyUri1);
            nextOntology.setOntologyVersionUri(nextOntologyVersionIri);
            
            final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
            
            Assert.assertNotNull("Could not find test file", inputStream);
            
            final Ontology storeOntology =
                    this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                            "application/rdf+xml", this.deleteOldVersionsTrue);
            
            Assert.assertNotNull(storeOntology);
            Assert.assertNotNull(storeOntology.getOntologyUri());
            Assert.assertNotNull(storeOntology.getOntologyVersionUri());
            
            Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
            Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
            
            Assert.assertEquals(nextOntologyVersionIri, storeOntology.getOntologyVersionUri());
            
            // verify that the ontology was stored correctly
            final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
            
            Assert.assertNotNull(ontologiesByUri);
            Assert.assertFalse(ontologiesByUri.isEmpty());
            Assert.assertEquals(1, ontologiesByUri.size());
        }
    }
    
    /**
     * Tests that a single null ontology cannot be stored using the dataHandler.storeOntology
     * method, without specifying that old versions must be deleted.
     */
    @Test
    public void testStoreOntologyNull()
    {
        final Ontology nextOntology = null;
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        try
        {
            this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                    "application/rdf+xml", this.deleteOldVersionsFalse);
            
            Assert.fail("Did not find expected IllegalArgumentException");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("Ontology must not be null"));
        }
    }
    
    /**
     * Tests that a single empty ontology cannot be stored using the dataHandler.storeOntology
     * method, without specifying that old versions must be deleted.
     */
    @Test
    public void testStoreOntologySingleEmptyDeleteFalse()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        try
        {
            this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                    "application/rdf+xml", this.deleteOldVersionsFalse);
            
            Assert.fail("Did not find expected IllegalArgumentException");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("Could not store ontology without an ontology URI"));
        }
    }
    
    /**
     * Tests that a single ontology missing its ontology URI cannot be stored using the
     * dataHandler.storeOntology method, without specifying that old versions must be deleted.
     */
    @Test
    public void testStoreOntologySingleNoOntologyUriDeleteFalse()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        try
        {
            this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                    "application/rdf+xml", this.deleteOldVersionsFalse);
            
            Assert.fail("Did not find expected IllegalArgumentException");
        }
        catch(final IllegalArgumentException iae)
        {
            Assert.assertTrue(iae.getMessage().contains("Could not store ontology without an ontology URI"));
        }
    }
    
    /**
     * Tests that a single valid ontology with no version can be stored using the
     * dataHandler.storeOntology method, without specifying that old versions must be deleted
     */
    @Test
    public void testStoreOntologySingleNoVersionDeleteFalse()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        // verify that the ontology was stored correctly
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
    }
    
    /**
     * Tests that a single valid ontology with no version URI can be stored using the
     * dataHandler.storeOntology method, while specifying that old versions must be deleted
     */
    @Test
    public void testStoreOntologySingleNoVersionDeleteTrue()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        // verify that the ontology was stored correctly
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
    }
    
    /**
     * Tests that a single valid ontology can be stored using the dataHandler.storeOntology method,
     * without specifying that old versions must be deleted.
     */
    @Test
    public void testStoreOntologySingleValidDeleteFalse()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsFalse);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        // verify that the ontology was stored correctly
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
    }
    
    /**
     * Tests that a single valid ontology can be stored using the dataHandler.storeOntology method,
     * while specifying that old versions must be deleted.
     */
    @Test
    public void testStoreOntologySingleValidDeleteTrue()
    {
        final Ontology nextOntology = this.getNewOntology();
        
        nextOntology.setOntologyUri(this.testOntologyUri1);
        nextOntology.setOntologyVersionUri(this.testOntologyVersionUri1);
        
        final InputStream inputStream = this.getClass().getResourceAsStream("/testOntology1-1.rdf");
        
        Assert.assertNotNull("Could not find test file", inputStream);
        
        final Ontology storeOntology =
                this.dataHandler.storeOntology(nextOntology, inputStream, "http://base.example.org/",
                        "application/rdf+xml", this.deleteOldVersionsTrue);
        
        Assert.assertNotNull(storeOntology);
        Assert.assertNotNull(storeOntology.getOntologyUri());
        Assert.assertNotNull(storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyUri1, storeOntology.getOntologyUri());
        Assert.assertNotSame(this.testOntologyUri1, storeOntology.getOntologyVersionUri());
        
        Assert.assertEquals(this.testOntologyVersionUri1, storeOntology.getOntologyVersionUri());
        
        // verify that the ontology was stored correctly
        final List<Ontology> ontologiesByUri = this.dataHandler.getOntologiesByUri(storeOntology.getOntologyUri());
        
        Assert.assertNotNull(ontologiesByUri);
        Assert.assertFalse(ontologiesByUri.isEmpty());
        Assert.assertEquals(1, ontologiesByUri.size());
    }
    
    /**
     * Test method for
     * {@link com.github.ansell.oas.data.DataHandler#updateAnnotation(com.github.ansell.oas.objects.Annotation)}
     * .
     */
    @Test
    public void testUpdateAnnotation()
    {
        final Annotation nextAnnotation = this.getNewAnnotation();
        
        nextAnnotation.setAnnotationUniqueId(this.testUniqueAnnotationId);
        nextAnnotation.setAnnotatedObjectType(this.testAnnotatedObjectType);
        nextAnnotation.setAnnotatedObjectUri(this.testAnnotatedObjectUri);
        nextAnnotation.setAnnotator(this.testAnnotator);
        nextAnnotation.setOntologyTermUri(this.testOntologyUri1);
        nextAnnotation.setTag(this.testTagLiteral);
        
        Assert.assertTrue(this.dataHandler.storeAnnotation(nextAnnotation));
        
        final Annotation retrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        // make sure the annotation was stored correctly before testing the deletion part
        AbstractTestUtils.testAnnotationsMatch(nextAnnotation, retrievedAnnotation);
        
        retrievedAnnotation.setOntologyTermUri(this.differentTestOntologyUri);
        
        Assert.assertTrue(this.dataHandler.updateAnnotation(retrievedAnnotation));
        
        final Annotation updatedRetrievedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
        
        AbstractTestUtils.testAnnotationsMatch(retrievedAnnotation, updatedRetrievedAnnotation);
        
        Assert.assertTrue(this.dataHandler.deleteAnnotation(this.testUniqueAnnotationId));
        
        try
        {
            final Annotation testDeletedAnnotation = this.dataHandler.getAnnotation(this.testUniqueAnnotationId);
            
            Assert.assertNull(
                    "Annotation was not deleted correctly, as the result of a getAnnotation on a deleted annotation should be a null value",
                    testDeletedAnnotation);
        }
        catch(final Exception ex)
        {
            Assert.assertFalse("Received an exception when trying to access a deleted annotation", true);
        }
    }
    
}
